//+------------------------------------------------------------------+
//|                                                Magic_exp2_en.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| The class provides assisting trade calculations                  |
//+------------------------------------------------------------------+
class CProvision
  {
protected:
   MqlTradeRequest   trades;// Pointer to the structure of requesting by OrderSend
public:
   int               TYPE(const double &v[]); // Determines type depending on MA indication
   double            pricetype(int type);     // Calculates the level of opening depending on the type 
   double            SLtype(int type);        // Calculates Stop Loss level depending on the type 
   double            TPtype(int type);        // Calculates Take Profit level depending on the type
   long              spread();                // Returns spread of a current symbol
   ulong             SendOrder(ENUM_ORDER_TYPE type,double volume);
  };
//+------------------------------------------------------------------+
//| SendOrder                                                        |
//+------------------------------------------------------------------+
ulong CProvision::SendOrder(ENUM_ORDER_TYPE type,double volume)
  {
   trades.action=TRADE_ACTION_DEAL;                // Type of the action performed
   trades.magic=magic;                             // Stamp of Expert Advisor (the magic number identifier) the magic variable is declared at the global scope
   mg.SetMagic_request(int(digital_name_),int(code_interaction_));
   trades.symbol          =_Symbol;                // Symbol name
   trades.volume          =volume;                 // Requested volume of a deal in lots
   trades.price           =pricetype((int)type);   // Price       
   trades.sl              =SLtype((int)type);      // Stop Loss level of an order
   trades.tp              =TPtype((int)type);      // Take Profit level of an order         
   trades.deviation=(int)spread();                 // Maximum acceptable deviation from a requested price
   trades.type=type;                               // Order type
   trades.type_filling=ORDER_FILLING_FOK;
   if(OrderSend(trades,res)){return(res.retcode);}
   return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CProvision::TYPE(const double &v[])// Determine type depending on MA indication
  {
   double t=v[0]-v[1];// Since a reverse numeration is used in MAs do it inversely
   if(t==0.0)t=1.0;// If the MA doesn't rise or fall then the result is "buy"
   return((int)(0.5*t/fabs(t)+0.5));// if the MA rises display 0(buy), if it falls display 1(sell)
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CProvision::pricetype(int type)// Calculate the level of opening depending on the type 
  {
   if(SymbolInfoTick(_Symbol,tick))
     {
      if(type==0)return(tick.ask);
      if(type==1)return(tick.bid);
     }
   return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CProvision::SLtype(int type)// Calculate Stop Loss level depending on the type
  {
   if(SymbolInfoTick(_Symbol,tick))
     {
      if(type==0)return(tick.bid-SL*SymbolInfoDouble(Symbol(),SYMBOL_POINT));
      if(type==1)return(tick.ask+SL*SymbolInfoDouble(Symbol(),SYMBOL_POINT));
     }
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CProvision::TPtype(int type)// Calculate Take Profit level depending on the type
  {
   if(SymbolInfoTick(_Symbol,tick))
     {
      if(type==0)return(tick.bid+TP*SymbolInfoDouble(Symbol(),SYMBOL_POINT));
      if(type==1)return(tick.ask-TP*SymbolInfoDouble(Symbol(),SYMBOL_POINT));
     }
   return(0);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
long CProvision::spread()// Calculate spread
  {
   return(SymbolInfoInteger(_Symbol,SYMBOL_SPREAD));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum Emagic
  {
   ENUM_DIGITAL_NAME,    // digital name of an Expert Advisor
   ENUM_CODE_INTERACTION,// Code of interaction
   ENUM_EXPERT_SYMBOL    // Symbol at which an Expert Advisor is launched
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct Smagic
  {
   ulong             magicnumber;      // Magic number in a assembled form as it is written in order
   int               digital_name;     // Digital name
   int               code_interaction; // Code of interaction
   int               expert_symbol;    // Symbol at which an Expert Advisor is launched
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct SPositionVirtualMagic
  {
   double            volume;
   ENUM_POSITION_TYPE type;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class Cmagic
  {
protected:
   Smagic            mag;
   SPositionVirtualMagic pvm;
public:
   ulong             SetMagic_request(int digital_name=0,int code_interaction=0);
   // func. returns the assembled magic number combined from input data
   ulong             SetMagic_result(ulong magicnumber);
   // func. receives the assembled magic number and divides it in accordance with assembling logic
   ulong             GetMagic_result(Emagic enum_);
   // func. receives the return identifier and returns the requested part of the assembled magic number
   string            sGetMagic_result(Emagic enum_);
   // func. receives the return identifier and returns the text interprepation of requested part of the assembled magic number
   double               cVOLUME(){return(pvm.volume);}
   // Returns the volume of virtual position of an Expert Advisor
   ENUM_POSITION_TYPE   cTYPE(){return(pvm.type);}
   // Returns the type of virtual position of an Expert Advisor
   bool              PositionVirtualMagic(//const ulong Magic,
                                          Emagic enum_,
                                          string symbol,
                                          datetime CurrentTime);
   // the method of calculation virt. position returns the presence or absence of virt. position
private:
   int               decodeMagic_result(int category);
   // func. divideds the magic number into three parts of three bits and returns the part the category points at
   int               symbolexpert();
   // Interpreter of symbols of an instrument into a digital code
   string            expertcode(int code);
   // Interpreter of digital code into written text (EAs)
   string            codeinterdescript(int code);
   // Interpreter of digital code into  written text (interaction)
   string            symbolexpert(int code);
   // Interpreter of digital code into symbols of an instrument
   void              prHistory_Deals(ulong &buf[],int HTD);
   // Fills the array of tickets
  };
//+-------------------------------------------------------------------------------------------+
//| Method of calculation of virt. position returns the presence or absence of virt. position |
//+-------------------------------------------------------------------------------------------+
bool  Cmagic::PositionVirtualMagic(Emagic enum_,
                                   string symbol,
                                   datetime CurrentTime)
  {
   ulong Magic=GetMagic_result(enum_);
   int DIGITS=(int)-log10(SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP)); if(DIGITS<0)DIGITS=0;
   ulong Dticket=0;
   int History_Total_Deals=-1;
   double volume=0,volume_BUY=0,volume_SELL=0;
   ulong DTicketbuf[];

   do
     {
      if(HistorySelect(0,TimeCurrent()))
        {
         History_Total_Deals=HistoryDealsTotal();
         prHistory_Deals(DTicketbuf,History_Total_Deals);
        }
      HistorySelect(0,TimeCurrent());
     }
   while(History_Total_Deals!=HistoryDealsTotal());

   for(int t=0;t<History_Total_Deals;t++)
     {
      Dticket=DTicketbuf[t];
      if(HistoryDealSelect(Dticket))
        {
         if(HistoryDealGetInteger(Dticket,DEAL_TIME)>=CurrentTime)
           {
            if(HistoryDealGetString(Dticket,DEAL_SYMBOL)==symbol)
              {
               SetMagic_result(HistoryDealGetInteger(Dticket,DEAL_MAGIC));
               if(GetMagic_result(enum_)==Magic)
                 {
                  if(HistoryDealGetInteger(Dticket,DEAL_TYPE)==DEAL_TYPE_BUY)
                    {
                     volume_BUY+=HistoryDealGetDouble(Dticket,DEAL_VOLUME);
                    }
                  else
                    {
                     if(HistoryDealGetInteger(Dticket,DEAL_TYPE)==DEAL_TYPE_SELL)
                       {
                        volume_SELL+=HistoryDealGetDouble(Dticket,DEAL_VOLUME);
                       }
                    }
                 }
              }
           }
        }
      else{HistorySelect(0,TimeCurrent());t--;}
      // if there is a fault, load history data and pass the step again
     }
   volume=NormalizeDouble(volume_BUY-volume_SELL,DIGITS);
   if(volume<0)pvm.type=POSITION_TYPE_SELL;
   else
     {
      if(volume>0)pvm.type=POSITION_TYPE_BUY;
     }
   pvm.volume=fabs(volume); //Print("#",magic," #",Magic," pvm.volume ",pvm.volume," pvm.type ",pvm.type);
   if(pvm.volume==0)return(false);
   else return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void  Cmagic::prHistory_Deals(ulong &buf[],int HTD)
  {
   ArrayResize(buf,HTD);
   for(int t=0;t<HTD;t++)
     {
      buf[t]=HistoryDealGetTicket(t);
     }
  }
//+-------------------------------------------------------------------+
//| funct. returns the assemble magic number combined from input data |
//+-------------------------------------------------------------------+
ulong Cmagic::SetMagic_request(int digital_name=0,int code_interaction=0)
  {
   if(digital_name>=1000)Print("Wrong digital name specified for Expert Advisor (greater than 1000)");
   if(code_interaction>=1000)Print("Wrong code of identification friend-stranger  (greater than 1000)");
   mag.digital_name     =digital_name;
   mag.code_interaction =code_interaction;
   mag.expert_symbol    =symbolexpert();
   mag.magicnumber      =mag.digital_name*(int)pow(1000,2)+mag.code_interaction*(int)pow(1000,1)+mag.expert_symbol;
   return(mag.magicnumber);
  }
//+-----------------------------------------------------------------------------------------------+
//| funct. receives the assembled magic number and divides it in accordance with assembling logic |
//+-----------------------------------------------------------------------------------------------+
ulong Cmagic::SetMagic_result(ulong magicnumber)
  {
   mag.magicnumber      =magicnumber;
   mag.expert_symbol    =decodeMagic_result(1);
   mag.code_interaction =decodeMagic_result(2);
   mag.digital_name     =decodeMagic_result(3);
   return(mag.magicnumber);
  }
//+------------------------------------------------------------------------------------------------------------+
//| funct. divides the magic number into three parts of three bits and returns the part the category points at |
//+------------------------------------------------------------------------------------------------------------+
int Cmagic::decodeMagic_result(int category)
  {
   string string_value=(string)mag.magicnumber;
   int rem=(int)MathMod(StringLen(string_value),3);
   if(rem!=0)
     {
      rem=3-rem;
      string srem="0";
      if(rem==2)srem="00";
      string_value=srem+string_value;
     }
   int start_pos=StringLen(string_value)-3*category;
   string value=StringSubstr(string_value,start_pos,3);
   return((int)StringToInteger(value));
  }
//+----------------------------------------------------------------------------------------------------+
//| funct. receives the return identifier and returns the requested part of the assembled magic number |
//+----------------------------------------------------------------------------------------------------+
ulong Cmagic::GetMagic_result(Emagic enum_)
  {
   switch(enum_)
     {
      case ENUM_DIGITAL_NAME     : return(mag.digital_name);     break;
      case ENUM_CODE_INTERACTION : return(mag.code_interaction); break;
      case ENUM_EXPERT_SYMBOL    : return(mag.expert_symbol);    break;
      default: return(mag.magicnumber); break;
     }
  }
//+------------------------------------------------------------------+
//| funct. receives the return identifier and returns the text       |
//| interpretation of requested part of the assembled magic number   |
//+------------------------------------------------------------------+
string Cmagic::sGetMagic_result(Emagic enum_)
  {
   switch(enum_)
     {
      case ENUM_DIGITAL_NAME     : return(expertcode(mag.digital_name));            break;
      case ENUM_CODE_INTERACTION : return(codeinterdescript(mag.code_interaction)); break;
      case ENUM_EXPERT_SYMBOL    : return(symbolexpert(mag.expert_symbol));         break;
      default: return((string)mag.magicnumber); break;
     }
  }
//+------------------------------------------------------------------+
//| Interpreter of symbols into a digital code                       |
//+------------------------------------------------------------------+
int Cmagic::symbolexpert()
  {
   if("EURUSD"==Symbol()){return(1);}
   if("GBPUSD"==Symbol()){return(2);}
   if("USDCHF"==Symbol()){return(3);}
   if("USDJPY"==Symbol()){return(4);}
   if("USDCAD"==Symbol()){return(5);}
   if("AUDUSD"==Symbol()){return(6);}
   if("AUDNZD"==Symbol()){return(7);}
   if("AUDCAD"==Symbol()){return(8);}
   if("AUDCHF"==Symbol()){return(9);}
   if("AUDJPY"==Symbol()){return(10);}
   if("CHFJPY"==Symbol()){return(11);}
   if("EURGBP"==Symbol()){return(12);}
   if("EURAUD"==Symbol()){return(13);}
   if("EURCHF"==Symbol()){return(14);}
   if("EURJPY"==Symbol()){return(15);}
   if("EURNZD"==Symbol()){return(16);}
   if("EURCAD"==Symbol()){return(17);}
   if("GBPCHF"==Symbol()){return(18);}
   if("GBPJPY"==Symbol()){return(19);}
   if("CADCHF"==Symbol()){return(20);}
   if("#AA"==Symbol()){return(21);}
   if("#AIG"==Symbol()){return(22);}
   if("#AXP"==Symbol()){return(23);}
   if("#BA"==Symbol()){return(24);}
   if("#BAC"==Symbol()){return(25);}
   if("#C"==Symbol()){return(26);}
   if("#CAT"==Symbol()){return(27);}
   if("#CVX"==Symbol()){return(28);}
   if("#DD"==Symbol()){return(29);}
   if("#DIS"==Symbol()){return(30);}
   if("#EK"==Symbol()){return(31);}
   if("#GE"==Symbol()){return(32);}
   if("#HD"==Symbol()){return(33);}
   if("#HON"==Symbol()){return(34);}
   if("#HPQ"==Symbol()){return(35);}
   if("#IBM"==Symbol()){return(36);}
   if("#INTC"==Symbol()){return(37);}
   if("#IP"==Symbol()){return(38);}
   if("#JNJ"==Symbol()){return(39);}
   if("#JPM"==Symbol()){return(40);}
   if("#KFT"==Symbol()){return(41);}
   if("#KO"==Symbol()){return(42);}
   if("#MCD"==Symbol()){return(43);}
   if("#MMM"==Symbol()){return(44);}
   if("#MO"==Symbol()){return(45);}
   if("#MRK"==Symbol()){return(46);}
   if("#MSFT"==Symbol()){return(47);}
   if("#PFE"==Symbol()){return(48);}
   if("#PG"==Symbol()){return(49);}
   if("#T"==Symbol()){return(50);}
   if("#TRV"==Symbol()){return(51);}
   if("#UTX"==Symbol()){return(52);}
   if("#VZ"==Symbol()){return(53);}
   if("#WMT"==Symbol()){return(54);}
   if("#XOM"==Symbol()){return(55);}
   if("NZDUSD"==Symbol()){return(56);}
   if("USDRUR"==Symbol()){return(57);}
   if("EURRUR"==Symbol()){return(58);}
   if("USDSEK"==Symbol()){return(59);}
   return(-1);
  }
//+------------------------------------------------------------------+
//| Interpretor of digital code into symbols of an instrument        |
//+------------------------------------------------------------------+
string Cmagic::symbolexpert(int code)
  {
   if(code==1){return("EURUSD");}
   if(code==2){return("GBPUSD");}
   if(code==3){return("USDCHF");}
   if(code==4){return("USDJPY");}
   if(code==5){return("USDCAD");}
   if(code==6){return("AUDUSD");}
   if(code==7){return("AUDNZD");}
   if(code==8){return("AUDCAD");}
   if(code==9){return("AUDCHF");}
   if(code==10){return("AUDJPY");}
   if(code==11){return("CHFJPY");}
   if(code==12){return("EURGBP");}
   if(code==13){return("EURAUD");}
   if(code==14){return("EURCHF");}
   if(code==15){return("EURJPY");}
   if(code==16){return("EURNZD");}
   if(code==17){return("EURCAD");}
   if(code==18){return("GBPCHF");}
   if(code==19){return("GBPJPY");}
   if(code==20){return("CADCHF");}
   if(code==21){return("#AA");}
   if(code==22){return("#AIG");}
   if(code==23){return("#AXP");}
   if(code==24){return("#BA");}
   if(code==25){return("#BAC");}
   if(code==26){return("#C");}
   if(code==27){return("#CAT");}
   if(code==28){return("#CVX");}
   if(code==29){return("#DD");}
   if(code==30){return("#DIS");}
   if(code==31){return("#EK");}
   if(code==32){return("#GE");}
   if(code==33){return("#HD");}
   if(code==34){return("#HON");}
   if(code==35){return("#HPQ");}
   if(code==36){return("#IBM");}
   if(code==37){return("#INTC");}
   if(code==38){return("#IP");}
   if(code==39){return("#JNJ");}
   if(code==40){return("#JPM");}
   if(code==41){return("#KFT");}
   if(code==42){return("#KO");}
   if(code==43){return("#MCD");}
   if(code==44){return("#MMM");}
   if(code==45){return("#MO");}
   if(code==46){return("#MRK");}
   if(code==47){return("#MSFT");}
   if(code==48){return("#PFE");}
   if(code==49){return("#PG");}
   if(code==50){return("#T");}
   if(code==51){return("#TRV");}
   if(code==52){return("#UTX");}
   if(code==53){return("#VZ");}
   if(code==54){return("#WMT");}
   if(code==55){return("#XOM");}
   if(code==56){return("NZDUSD");}
   if(code==57){return("USDRUR");}
   if(code==58){return("EURRUR");}
   if(code==59){return("USDSEK");}
   return("NONE");
  }
//+------------------------------------------------------------------+
//| Interpretor of digital code into written text(EAs)               |
//+------------------------------------------------------------------+
string Cmagic::expertcode(int code)
  {
   if(code==1){return("MACD");}
   if(code==2){return("MA20");}
   if(code==3){return("-\\-");}
   if(code==4){return("-\\-");}
   if(code==5){return("-\\-");}
   if(code==6){return("-\\-");}
   if(code==7){return("-\\-");}
   if(code==8){return("-\\-");}
   if(code==9){return("-\\-");}
   if(code==10){return("-\\-");}
   if(code==11){return("-\\-");}
   if(code==12){return("-\\-");}
   if(code==13){return("-\\-");}
   if(code==14){return("-\\-");}
   if(code==15){return("-\\-");}
   return("NONE");
  }
//+------------------------------------------------------------------+
//| Interpretor of digital code into written text (interaction)      |
//+------------------------------------------------------------------+
string Cmagic::codeinterdescript(int code)
  {
   if(code==1){return("MACD+MA20");}
   if(code==2){return("-\\-");}
   if(code==3){return("-\\-");}
   if(code==4){return("-\\-");}
   if(code==5){return("-\\-");}
   if(code==6){return("-\\-");}
   if(code==7){return("-\\-");}
   if(code==8){return("-\\-");}
   if(code==9){return("-\\-");}
   if(code==10){return("-\\-");}
   if(code==11){return("-\\-");}
   if(code==12){return("-\\-");}
   if(code==13){return("-\\-");}
   if(code==14){return("-\\-");}
   if(code==15){return("-\\-");}
   return("NONE");
  }
//+------------------------------------------------------------------+
//|XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX|
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Code of the Expert Advisor                                       |
//+------------------------------------------------------------------+

//--- input parameters
input ulong              digital_name_       =4;           // Digital name of Expert Advisor
input ulong              code_interaction_   =1;           // Code of interaction
input Emagic             _enum               =0;           // Model of magic number  
input int                SL                  =300;         // Stop Loss
input int                TP                  =1000;        // Take Profit
input int                MA_Period           =25;          // MA period
input double             lot                 =0.4;         // Volume of position
input int                MA_shift            =0;           // Shift of indicator
input ENUM_MA_METHOD     MA_smooth           =MODE_SMA;    // Smoothing type
input ENUM_APPLIED_PRICE price               =PRICE_OPEN;  // Price type 

//--- we will store the indicator's handle
int MA_handle,type_MA;
ulong rezult;
static ulong magic;
double v[2];
datetime  CurrentTime;// The variable stores the time of start of the Expert Advisor

MqlTradeResult  res;   // Pointer to the structure of responding by OrderSend
MqlTick         tick;  // Pointer to the structure of last market information
CProvision prov;
Cmagic mg;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   magic=mg.SetMagic_request(int(digital_name_),int(code_interaction_));
// Stamp of Expert Advisor (the magic number identifier) the magic variable is declared at the global scope
// used in int CProvision::SendOrder(ENUM_ORDER_TYPE type,double volume)
   CurrentTime=TimeCurrent();// The variable stores the time of start of the Expert Advisor
//--- Create the indicator's handle
   MA_handle=iMA(Symbol(),0,MA_Period,MA_shift,MA_smooth,price);
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   if(CopyBuffer(MA_handle,0,0,2,v)<=0)
     {Print("#",magic,"Error of copying");return;}
   type_MA=prov.TYPE(v); // Determine type depending on MA indication
   mg.SetMagic_result(magic);// put the information into the structure
   if(mg.PositionVirtualMagic(_enum,_Symbol,CurrentTime))// If three is an open position 
     {
      if((int)mg.cTYPE()!=type_MA)// Check if it is time to close
        {
         mg.SetMagic_result(magic);// put the information into the structure
         Print("#",mg.GetMagic_result(_enum),"Position by magic number has volume ",mg.cVOLUME(),
               " reverse position of type ",(int)mg.cTYPE()," by ",type_MA);
         //cpvm.cVOLUME() - volume of virtual position
         rezult=prov.SendOrder((ENUM_ORDER_TYPE)type_MA,mg.cVOLUME()+lot);// reverse position
         if(rezult!=-1)Print("#",magic," Code of the operation result ",rezult," volume ",res.volume);
         else{Print("#",magic,"Error",GetLastError()); return;}
        }
     }
   else // If there is no open position then open
     {
      Print("#",magic,"Position by magic number has volume  ",mg.cVOLUME()," open position of type",type_MA);
      rezult=prov.SendOrder((ENUM_ORDER_TYPE)type_MA,lot);// Open position 
      if(rezult!=-1)Print("#",magic," Code of the operation result ",rezult," volume ",res.volume);
      else{Print("#",magic,"Error",GetLastError()); return;}
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 

//text789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
//cod456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_12
